@@ -1,17 +1,17 @@
-#ifdef _CH_
-#pragma package <opencv>
-#endif
+#include <opencv2/imgproc/imgproc_c.h>
+#include "opencv2/highgui/highgui.hpp"
 
-#define CV_NO_BACKWARD_COMPATIBILITY
-
-#ifndef _EiC
-#include "cv.h"
-#include "highgui.h"
 #include <stdio.h>
-#endif
-
-/* the script demostrates iterative construction of
-   delaunay triangulation and voronoi tesselation */
+void help()
+{
+	printf("\nThis program demostrates iterative construction of\n"
+			"delaunay triangulation and voronoi tesselation.\n"
+			"It draws a random set of points in an image and then delaunay triangulates them.\n"
+            "Usage: \n"
+            "./delaunay \n"
+			"\nThis program builds the traingulation interactively, you may stop this process by\n"
+			"hitting any key.\n");
+}
 
 CvSubdiv2D* init_delaunay( CvMemStorage* storage,
                            CvRect rect )
@@ -233,6 +233,7 @@ int main( int argc, char** argv )
 #ifdef _MSC_VER
     argc, argv;
 #endif
+    help();
     run();
     return 0;
 }
