@@ -1,4 +1,5 @@
-import cv
+import cv2.cv as cv
+import time
 
 cv.NamedWindow("camera", 1)
 
